-- **************************************************
-- MEDICAL NOTE:
-- So you've decided to look at my code? 
-- Then you might suffer from the following affliction: http://en.wikipedia.org/wiki/Masochistic_personality_disorder
--
-- Please consult your physician before attempting to make sense of the following madness.
-- **************************************************


-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_round_edges"

-- **************************************************
-- General information about this script
-- **************************************************

DS_round_edges = {}


function DS_round_edges:Name()
	return "Round edges"
end

function DS_round_edges:Version()
	return "0.9"
end

function DS_round_edges:Description()
	return "Inserts circles over points"
end

function DS_round_edges:Creator()
	return "dauid.com"
end

function DS_round_edges:UILabel()
	return"Round edges"
end

-- **************************************************
-- Options dialog
-- **************************************************
local DS_Dialog = {}

DS_Dialog.UPDATE = MOHO.MSG_BASE

function DS_Dialog:new(moho)
	local d = LM.GUI.SimpleDialog("Options", DS_Dialog)
	local l = d:GetLayout()

	d.moho = moho
	
	selectBox = LM.GUI.CheckBox("Only work on selected points")
	l:AddChild(selectBox, LM.GUI.ALIGN_LEFT)
	
	fillBox = LM.GUI.CheckBox("One fill for all shapes")
	l:AddChild(fillBox, LM.GUI.ALIGN_LEFT)
	
	scolBox = LM.GUI.CheckBox("Use current stroke color (otherwise fill color used)")
	l:AddChild(scolBox, LM.GUI.ALIGN_LEFT)
	scolBox:SetValue(true)
	
	
	boneBox = LM.GUI.CheckBox("Bind to same bone as point")
	l:AddChild(boneBox, LM.GUI.ALIGN_LEFT)
	boneBox:SetValue(true)
	
	connectBox = LM.GUI.CheckBox("Connect circle with point")
	l:AddChild(connectBox, LM.GUI.ALIGN_LEFT)
	connectBox:SetValue(true)

	
	return d
end


function DS_Dialog:OnOK()

doFill=fillBox:Value() 		--Get values from checkboxes so we know what to do and what not to do
doSelect=selectBox:Value()
doBone=boneBox:Value()
doConnect=connectBox:Value()
doScol=scolBox:Value()

end

-- **************************************************
-- The guts of this script
-- **************************************************

function DS_round_edges:Run(moho)

local dlog = DS_Dialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
	
	
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	
	local Xvalues={} 	--Table for storing X positions of points
	local Yvalues={}  	--Table for storing Y positions of points
	local WidthVal={}  	--Table for storing width value of points
	local pBone={}  	--Table for storing parent bone of points so that we can bind the new points to the same bone
	local pId={}  		--Table for storing point ID. Used so that we can unselect the original point before creating a shape. If selected, bad stuff happens. Butt stuff...
	
	for i = 0, mesh:CountPoints() - 1 do --Go through all the points
		local pt = mesh:Point(i)
			if (doSelect==false) then 				--If we do NOT want to work only on selected points...
				if (pt:IsEndpoint()) then			--Then work on every end point.
					pt:SetCurvature(MOHO.PEAKED, 0)	--Set curvature to peaked. If this is not done on end points and the created circles connect to the point, the line will change shape
					table.insert(Xvalues, pt.fPos.x)
					table.insert(Yvalues, pt.fPos.y)
					table.insert(pBone, pt.fParent)
					table.insert(pId, i)
					if (pt.fWidth.value<0) then
					table.insert(WidthVal, 0.0055555556900799) --If the point doesn't have a width value, use this. Equal to a width of 4 in AS.
					else
					table.insert(WidthVal, pt.fWidth.value)
					end
				end
			else
				if (pt.fSelected) then		--Stuff that happens if we DO want to only work on selected points
					if (pt:IsEndpoint()) then		
					pt:SetCurvature(MOHO.PEAKED, 0)
					end
					table.insert(Xvalues, pt.fPos.x)
					table.insert(Yvalues, pt.fPos.y)
					table.insert(pBone, pt.fParent)
					table.insert(pId, i)
					if (pt.fWidth.value<0) then
					table.insert(WidthVal, 0.0055555556900799) --If the point doesn't have a width value, use this. Equal to a width of 4 in AS.
					else
					table.insert(WidthVal, pt.fWidth.value)
					end
				end
			
			end
	end
	local pointsBefore=mesh:CountPoints() 	--If we get the number of points before anything new is created, we can calculate the ID's of all the new points. 
	for i,v in ipairs(Xvalues) do  			--Go through the tables and do stuff.
		if (WidthVal[i]>0)then				--If the width of a point is zero, no reason to create anything
			self:BuildPolygon(mesh, Xvalues[i], Yvalues[i], WidthVal[i], pBone[i], pId[i]) --Values sent to build polygon function: mesh, x position, y position, size, parent bone, point ID
				if (doFill==false) then		--If every shape should have a separate fill then create shapes at this stage
				local id=moho:CreateShape(true)
				mesh:Shape(id).fHasOutline = false
					if (doScol==true) then		--If we want to get the current stroke color and use as fill
					local strokeColor = LM.ColorVector:new_local() 
					StrokeColor=mesh:Shape(id).fMyStyle.fLineCol:GetValue(moho.frame)
					mesh:Shape(id).fMyStyle.fFillCol:SetValue(moho.frame, StrokeColor)
					end
				end
			mesh:SelectNone() --Weird stuff happens unless we deselect all points before creating new ones.
		end
	end
	local pointsAfter=mesh:CountPoints()
	for g=pointsBefore, pointsAfter do			--This code is used to select all new points created
		for i = 0, mesh:CountPoints() - 1 do
			local pt = mesh:Point(i)
				if (i==g) then				
					pt.fSelected=true
				end
		end
	end
	if (doFill==true) then 			--If we want all the points to have the same fill, now is the time to create a shape.
	local id=moho:CreateShape(true)
	mesh:Shape(id).fHasOutline = false
		if (doScol==true) then
		local strokeColor = LM.ColorVector:new_local() 
		StrokeColor=mesh:Shape(id).fMyStyle.fLineCol:GetValue(moho.frame)
		mesh:Shape(id).fMyStyle.fFillCol:SetValue(moho.frame, StrokeColor)
		end
	end
	mesh:AddGroup("Round edges") --Add all the new points to a group called Round edges. If you run the script again only the points created that time will be in the group.

end

function DS_round_edges:BuildPolygon(mesh, theX, theY, size, theBone, theId) --The function for drawing the damn things


local oldNumPoints = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	mesh:SelectNone()
	
	
	v.x=theX-size
	v.y=theY
local pointOne=(mesh:AddLonePoint(v, 0))
	v.x=theX
	v.y=theY+size
local pointTwo=(mesh:AppendPoint(v, 0))

	v.x=theX+size
	v.y=theY
local pointThree=(mesh:AppendPoint(v, 0))

	v.x=theX
	v.y=theY-size
local pointFour=(mesh:AppendPoint(v, 0))

	v.x=theX-size
	v.y=theY
local pointFive=(mesh:AppendPoint(v, 0))



	mesh:WeldPoints(oldNumPoints, oldNumPoints+4, 0)
	--------------------------------------------------------------------------
	if (doConnect==true) then 		--If we want the circles to be connected to the original points.
		v.x=theX
		v.y=theY
		local pointSix=(mesh:AddLonePoint(v, 0))
		
		v.x=theX-size
		v.y=theY
		local pointSeven=(mesh:AppendPoint(v, 0))

		mesh:WeldPoints(oldNumPoints+3, oldNumPoints+5, 0)
		mesh:WeldPoints(oldNumPoints+3, theId, 0)
	end
--------------------------------------------------------------------------------------------

	for i = 0, 3 do
		mesh:Point(oldNumPoints+i):SetCurvature(0.391379, 0)
		mesh:Point(oldNumPoints+i).fSelected=true
		if (doBone==true) then
		mesh:Point(oldNumPoints+i).fParent=theBone --Bind the new points to the same bone as the original point. Or no bone if that's the case.
		end
		
	end
	mesh:Point(theId).fSelected=false --Deselect the original point, otherwise crap will happen when we create shapes.
end